package main;

import com.mchange.v2.c3p0.ComboPooledDataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import main.Config;

public class L2DatabaseFactory {

   static Logger _log = Logger.getLogger(L2DatabaseFactory.class.getName());
   private static L2DatabaseFactory _instance;
   private L2DatabaseFactory.ProviderType _providerType;
   private ComboPooledDataSource _source;


   public L2DatabaseFactory() throws SQLException {
      try {
         if(Config.DATABASE_MAX_CONNECTIONS < 2) {
            Config.DATABASE_MAX_CONNECTIONS = 2;
            _log.warning("A minimum of " + Config.DATABASE_MAX_CONNECTIONS + " db connections are required.");
         }

         this._source = new ComboPooledDataSource();
         this._source.setAutoCommitOnClose(true);
         this._source.setInitialPoolSize(10);
         this._source.setMinPoolSize(10);
         this._source.setMaxPoolSize(Math.max(10, Config.DATABASE_MAX_CONNECTIONS));
         this._source.setAcquireRetryAttempts(0);
         this._source.setAcquireRetryDelay(500);
         this._source.setCheckoutTimeout(0);
         this._source.setAcquireIncrement(5);
         this._source.setAutomaticTestTable("connection_test_table");
         this._source.setTestConnectionOnCheckin(false);
         this._source.setIdleConnectionTestPeriod(3600);
         this._source.setMaxIdleTime(Config.DATABASE_MAX_IDLE_TIME);
         this._source.setMaxStatementsPerConnection(100);
         this._source.setBreakAfterAcquireFailure(false);
         this._source.setDriverClass(Config.DATABASE_DRIVER);
         this._source.setJdbcUrl(Config.DATABASE_URL);
         this._source.setUser(Config.DATABASE_LOGIN);
         this._source.setPassword(Config.DATABASE_PASSWORD);
         this._source.getConnection().close();
         if(Config.DEBUG) {
            _log.fine("Database Connection Working");
         }

         if(Config.DATABASE_DRIVER.toLowerCase().contains("microsoft")) {
            this._providerType = L2DatabaseFactory.ProviderType.MsSql;
         } else {
            this._providerType = L2DatabaseFactory.ProviderType.MySql;
         }

      } catch (SQLException var2) {
         if(Config.DEBUG) {
            _log.fine("Database Connection FAILED");
         }

         throw var2;
      } catch (Exception var3) {
         if(Config.DEBUG) {
            _log.fine("Database Connection FAILED");
         }

         throw new SQLException("could not init DB connection:" + var3);
      }
   }

   public final String prepQuerySelect(String[] fields, String tableName, String whereClause, boolean returnOnlyTopRecord) {
      String msSqlTop1 = "";
      String mySqlTop1 = "";
      if(returnOnlyTopRecord) {
         if(this.getProviderType() == L2DatabaseFactory.ProviderType.MsSql) {
            msSqlTop1 = " Top 1 ";
         }

         if(this.getProviderType() == L2DatabaseFactory.ProviderType.MySql) {
            mySqlTop1 = " Limit 1 ";
         }
      }

      String query = "SELECT " + msSqlTop1 + this.safetyString(fields) + " FROM " + tableName + " WHERE " + whereClause + mySqlTop1;
      return query;
   }

   public void shutdown() {
      try {
         this._source.close();
      } catch (Exception var3) {
         _log.log(Level.INFO, "", var3);
      }

      try {
         this._source = null;
      } catch (Exception var2) {
         _log.log(Level.INFO, "", var2);
      }

   }

   public final String safetyString(String ... whatToCheck) {
      char braceLeft;
      char braceRight;
      if(this.getProviderType() == L2DatabaseFactory.ProviderType.MsSql) {
         braceLeft = 91;
         braceRight = 93;
      } else {
         braceLeft = 96;
         braceRight = 96;
      }

      int length = 0;
      String[] var8 = whatToCheck;
      int var7 = whatToCheck.length;

      for(int word = 0; word < var7; ++word) {
         String sbResult = var8[word];
         length += sbResult.length() + 4;
      }

      StringBuilder var10 = new StringBuilder(length);
      String[] var9 = whatToCheck;
      int var12 = whatToCheck.length;

      for(var7 = 0; var7 < var12; ++var7) {
         String var11 = var9[var7];
         if(var10.length() > 0) {
            var10.append(", ");
         }

         var10.append(braceLeft);
         var10.append(var11);
         var10.append(braceRight);
      }

      return var10.toString();
   }

   public static L2DatabaseFactory getInstance() throws SQLException {
      Class var0 = L2DatabaseFactory.class;
      synchronized(L2DatabaseFactory.class) {
         if(_instance == null) {
            _instance = new L2DatabaseFactory();
         }
      }

      return _instance;
   }

   public Connection getConnection() {
      Connection con = null;

      while(con == null) {
         try {
            con = this._source.getConnection();
         } catch (SQLException var3) {
            _log.warning("L2DatabaseFactory: getConnection() failed, trying again " + var3);
         }
      }

      return con;
   }

   public int getBusyConnectionCount() throws SQLException {
      return this._source.getNumBusyConnectionsDefaultUser();
   }

   public int getIdleConnectionCount() throws SQLException {
      return this._source.getNumIdleConnectionsDefaultUser();
   }

   public final L2DatabaseFactory.ProviderType getProviderType() {
      return this._providerType;
   }

   public static enum ProviderType {

      MySql,
      MsSql;
      // $FF: synthetic field
      private static final L2DatabaseFactory.ProviderType[] ENUM$VALUES = new L2DatabaseFactory.ProviderType[]{MySql, MsSql};


   }
}
